@@ -18,8 +18,9 @@ from IPython.core.profiledir import ProfileDir
 from ipython_genutils.importstring import import_item
 from ipython_genutils.py3compat import string_types
 from IPython.utils.sysinfo import num_cpus
-from traitlets import (Integer, Unicode, Bool, CFloat, Dict, List, Any,
-                                        DottedObjectName)
+from traitlets import (
+    Integer, Unicode, Bool, CFloat, Dict, List, Any, DottedObjectName, observe
+)
 
 from .baseapp import (
     BaseParallelApplication,
@@ -57,6 +58,10 @@ ipcluster engines -h  # show the help string for the engines subcmd
 _start_examples = """
 ipython profile create mycluster --parallel # create mycluster profile
 ipcluster start --profile=mycluster --n=4   # start mycluster with 4 nodes
+
+# args to `ipcluster start` after `--` are passed through to the controller
+# see `ipcontroller -h` for options
+ipcluster start -- --sqlitedb
 """
 
 _stop_examples = """
@@ -238,11 +243,13 @@ class IPClusterEngines(BaseParallelApplication):
         CPU on your machine""")
 
     engine_launcher = Any(config=True, help="Deprecated, use engine_launcher_class")
-    def _engine_launcher_changed(self, name, old, new):
-        if isinstance(new, string_types):
+
+    @observe('engine_launcher')
+    def _engine_launcher_changed(self, change):
+        if isinstance(change['new'], string_types):
             self.log.warn("WARNING: %s.engine_launcher is deprecated as of 0.12,"
                     " use engine_launcher_class" % self.__class__.__name__)
-            self.engine_launcher_class = new
+            self.engine_launcher_class = change['new']
     engine_launcher_class = DottedObjectName('LocalEngineSetLauncher',
         config=True,
         help="""The class for launching a set of Engines. Change this value
@@ -284,13 +291,14 @@ class IPClusterEngines(BaseParallelApplication):
         Not available on Windows.
         """)
 
-    def _daemonize_changed(self, name, old, new):
-        if new:
+    @observe('daemonize')
+    def _daemonize_changed(self, change):
+        if change['new']:
             self.log_to_file = True
 
     early_shutdown = Integer(30, config=True, help="The timeout (in seconds)")
     _stopping = False
-    
+
     aliases = Dict(engine_aliases)
     flags = Dict(engine_flags)
 
@@ -325,7 +333,7 @@ class IPClusterEngines(BaseParallelApplication):
         if self.engine_launcher.running:
             self.log.info("Engines appear to have started successfully")
             self.early_shutdown = 0
-    
+
     def start_engines(self):
         # Some EngineSetLaunchers ignore `n` and use their own engine count, such as SSH:
         n = getattr(self.engine_launcher, 'engine_count', self.n)
@@ -343,21 +351,21 @@ class IPClusterEngines(BaseParallelApplication):
         if self.early_shutdown and not self._stopping:
             self.log.error("""
             Engines shutdown early, they probably failed to connect.
-            
+
             Check the engine log files for output.
-            
+
             If your controller and engines are not on the same machine, you probably
             have to instruct the controller to listen on an interface other than localhost.
-            
+
             You can set this by adding "--ip='*'" to your ControllerLauncher.controller_args.
-            
+
             Be sure to read our security docs before instructing your controller to listen on
             a public interface.
             """)
             self.stop_launchers()
-        
+
         return self.engines_stopped(r)
-    
+
     def engines_stopped(self, r):
         return self.loop.stop()
 
@@ -402,7 +410,7 @@ class IPClusterEngines(BaseParallelApplication):
         if self.daemonize:
             if os.name=='posix':
                 daemonize()
-        
+
         self.loop.add_callback(self.start_engines)
         # Now write the new pid file AFTER our new forked pid is active.
         # self.write_pid_file()
@@ -422,6 +430,7 @@ start_aliases.update(dict(
     delay='IPClusterStart.delay',
     controller='IPClusterStart.controller_launcher_class',
     ip='IPClusterStart.controller_ip',
+    location='IPClusterStart.controller_location',
 ))
 start_aliases['clean-logs'] = 'IPClusterStart.clean_logs'
 
@@ -443,15 +452,23 @@ class IPClusterStart(IPClusterEngines):
 
     delay = CFloat(1., config=True,
         help="delay (in s) between starting the controller and the engines")
-    
+
     controller_ip = Unicode(config=True, help="Set the IP address of the controller.")
+    controller_location = Unicode(config=True,
+        help="""Set the location (hostname or ip) of the controller.
+        
+        This is used by engines and clients to locate the controller
+        when the controller listens on all interfaces
+        """)
     controller_launcher = Any(config=True, help="Deprecated, use controller_launcher_class")
-    def _controller_launcher_changed(self, name, old, new):
-        if isinstance(new, string_types):
+
+    @observe('controller_launcher')
+    def _controller_launcher_changed(self, change):
+        if isinstance(change['new'], string_types):
             # old 0.11-style config
             self.log.warn("WARNING: %s.controller_launcher is deprecated as of 0.12,"
                     " use controller_launcher_class" % self.__class__.__name__)
-            self.controller_launcher_class = new
+            self.controller_launcher_class = change['new']
     controller_launcher_class = DottedObjectName('LocalControllerLauncher',
         config=True,
         help="""The class for launching a Controller. Change this value if you want
@@ -459,7 +476,7 @@ class IPClusterStart(IPClusterEngines):
 
         Each launcher class has its own set of configuration options, for making sure
         it will work in your environment.
-        
+
         Note that using a batch launcher for the controller *does not* put it
         in the same batch job as the engines, so they will still start separately.
 
@@ -495,14 +512,30 @@ class IPClusterStart(IPClusterEngines):
 
     def init_launchers(self):
         self.controller_launcher = self.build_launcher(self.controller_launcher_class, 'Controller')
+
+        controller_args = getattr(self.controller_launcher, 'controller_args', None)
+        if controller_args is None:
+            def add_args(args):
+                # only some Launchers support modifying controller args
+                self.log.warning(
+                    "Not adding controller args %s. "
+                    "controller_args passthrough is not supported by %s",
+                    args, self.controller_launcher.__class__.__name__,
+                )
+        else:
+            add_args = controller_args.extend
         if self.controller_ip:
-            self.controller_launcher.controller_args.append('--ip=%s' % self.controller_ip)
+            add_args(['--ip=%s' % self.controller_ip])
+        if self.controller_location:
+            add_args(['--location=%s' % self.controller_location])
+        if self.extra_args:
+            add_args(self.extra_args)
         self.engine_launcher = self.build_launcher(self.engine_launcher_class, 'EngineSet')
-    
+
     def engines_stopped(self, r):
         """prevent parent.engines_stopped from stopping everything on engine shutdown"""
         pass
-    
+
     def start_controller(self):
         self.log.info("Starting Controller with %s", self.controller_launcher_class)
         self.controller_launcher.on_stop(self.stop_launchers)
@@ -550,7 +583,7 @@ class IPClusterStart(IPClusterEngines):
         if self.daemonize:
             if os.name=='posix':
                 daemonize()
-        
+
         def start():
             self.start_controller()
             self.loop.add_timeout(self.loop.time() + self.delay, self.start_engines)
@@ -572,24 +605,28 @@ class IPClusterStart(IPClusterEngines):
 
 class IPClusterNBExtension(BaseIPythonApplication):
     """Enable/disable ipcluster tab extension in Jupyter notebook"""
-    
+
     name = 'ipcluster-nbextension'
-    
+
     description = """Enable/disable IPython clusters tab in Jupyter notebook
-    
+
     for Jupyter Notebook >= 4.2, you can use the new nbextension API:
-    
+
     jupyter serverextension enable --py ipyparallel
     jupyter nbextension install --py ipyparallel
     jupyter nbextension enable --py ipyparallel
     """
-    
+
     examples = """
     ipcluster nbextension enable
     ipcluster nbextension disable
     """
     version = __version__
-    
+    user = Bool(False, help="Apply the operation only for the given user").tag(
+        config=True)
+    flags = Dict({'user': ({'IPClusterNBExtension': {'user': True}},
+                           'Apply the operation only for the given user')})
+
     def start(self):
         from ipyparallel.nbextension.install import install_extensions
         if len(self.extra_args) != 1:
@@ -597,10 +634,10 @@ class IPClusterNBExtension(BaseIPythonApplication):
         action = self.extra_args[0].lower()
         if action == 'enable':
             print("Enabling IPython clusters tab")
-            install_extensions(enable=True)
+            install_extensions(enable=True, user=self.user)
         elif action == 'disable':
             print("Disabling IPython clusters tab")
-            install_extensions(enable=False)
+            install_extensions(enable=False, user=self.user)
         else:
             self.exit("Must specify 'enable' or 'disable', not '%s'" % action)
 
@@ -626,7 +663,9 @@ class IPClusterApp(BaseIPythonApplication):
 
     def start(self):
         if self.subapp is None:
-            print("No subcommand specified. Must specify one of: %s"%(self.subcommands.keys()))
+            keys = ', '.join("'{}'".format(key) for
+                             key in self.subcommands.keys())
+            print("No subcommand specified. Must specify one of: %s" % keys)
             print()
             self.print_description()
             self.print_subcommands()
@@ -638,4 +677,3 @@ launch_new_instance = IPClusterApp.launch_instance
 
 if __name__ == '__main__':
     launch_new_instance()
-
