@@ -9,6 +9,7 @@ import sys
 import time
 from concurrent.futures import Future
 from datetime import datetime
+import threading
 from threading import Event
 try:
     from queue import Queue
@@ -16,15 +17,16 @@ except ImportError: # py2
     from Queue import Queue
 
 from decorator import decorator
-from tornado.gen import multi_future
 import zmq
 from zmq import MessageTracker
 
+from IPython import get_ipython
 from IPython.core.display import clear_output, display, display_pretty
 from ipyparallel import error
+from ipyparallel.util import utcnow, compare_datetimes
 from ipython_genutils.py3compat import string_types
 
-from .futures import MessageFuture
+from .futures import MessageFuture, multi_future
 
 def _raw_text(s):
     display_pretty(s, raw=True)
@@ -43,7 +45,8 @@ def check_ready(f, self, *args, **kwargs):
     return f(self, *args, **kwargs)
 
 _metadata_keys = []
-_FOREVER = int(1e8) # a long time because some infinite timeouts are uninterruptible
+# threading.TIMEOUT_MAX new in 3.2
+_FOREVER = getattr(threading, 'TIMEOUT_MAX', int(1e6))
 
 class AsyncResult(Future):
     """Class for representing results of non-blocking calls.
@@ -90,7 +93,7 @@ class AsyncResult(Future):
         else:
             self._metadata = [self._client.metadata[id] for id in self.msg_ids]
         self._init_futures()
-    
+
     def _init_futures(self):
         """Build futures for results and output; hook up callbacks"""
         if not self._children:
@@ -319,12 +322,13 @@ class AsyncResult(Future):
             if timeout and timeout < 0:
                 # Event doesn't like timeout < 0
                 timeout = None
+            elif timeout == 0:
+                raise error.TimeoutError("Still waiting to be sent")
             # wait for Future to indicate send having been called,
             # which means MessageTracker is ready.
             tic = time.time()
             if not self._sent_event.wait(timeout):
                 raise error.TimeoutError("Still waiting to be sent")
-                return False
             if timeout:
                 timeout = max(0, timeout - (time.time() - tic))
         try:
@@ -371,6 +375,8 @@ class AsyncResult(Future):
     @staticmethod
     def _wait_for_child(child, evt, timeout=_FOREVER):
         """Wait for a child to be done"""
+        if child.done():
+            return
         evt.clear()
         child.add_done_callback(lambda f: evt.set())
         evt.wait(timeout)
@@ -415,7 +421,7 @@ class AsyncResult(Future):
         end : one or more datetime objects (e.g. ar.received)
         start_key : callable
             Function to call on `start` to extract the relevant
-            entry [defalt: min]
+            entry [default: min]
         end_key : callable
             Function to call on `end` to extract the relevant
             entry [default: max]
@@ -434,7 +440,7 @@ class AsyncResult(Future):
             # handle single_result AsyncResults, where ar.stamp is single object,
             # not a list
             end = end_key(end)
-        return (end - start).total_seconds()
+        return compare_datetimes(end, start).total_seconds()
         
     @property
     def progress(self):
@@ -451,13 +457,13 @@ class AsyncResult(Future):
         if self.ready():
             return self.wall_time
         
-        now = submitted = datetime.now()
+        now = submitted = utcnow()
         for msg_id in self.msg_ids:
             if msg_id in self._client.metadata:
                 stamp = self._client.metadata[msg_id]['submitted']
                 if stamp and stamp < submitted:
                     submitted = stamp
-        return (now-submitted).total_seconds()
+        return compare_datetimes(now, submitted).total_seconds()
     
     @property
     @check_ready
@@ -468,7 +474,7 @@ class AsyncResult(Future):
         """
         t = 0
         for md in self._metadata:
-            t += (md['completed'] - md['started']).total_seconds()
+            t += compare_datetimes(md['completed'], md['started']).total_seconds()
         return t
     
     @property
@@ -498,9 +504,8 @@ class AsyncResult(Future):
     
     def _republish_displaypub(self, content, eid):
         """republish individual displaypub content dicts"""
-        try:
-            ip = get_ipython()
-        except NameError:
+        ip = get_ipython()
+        if ip is None:
             # displaypub is meaningless outside IPython
             return
         md = content['metadata'] or {}
@@ -524,10 +529,7 @@ class AsyncResult(Future):
     def _display_single_result(self):
         self._display_stream(self.stdout)
         self._display_stream(self.stderr, file=sys.stderr)
-        
-        try:
-            get_ipython()
-        except NameError:
+        if get_ipython() is None:
             # displaypub is meaningless outside IPython
             return
         
@@ -586,11 +588,9 @@ class AsyncResult(Future):
                 self._display_stream(stdout, '[stdout:%i] ' % eid)
                 self._display_stream(stderr, '[stderr:%i] ' % eid, file=sys.stderr)
                 
-                try:
-                    get_ipython()
-                except NameError:
+                if get_ipython() is None:
                     # displaypub is meaningless outside IPython
-                    return 
+                    continue
                 
                 if outputs or execute_result is not None:
                     _raw_text('[output:%i]' % eid)
@@ -605,14 +605,12 @@ class AsyncResult(Future):
             # republish stdout:
             for eid,stdout in zip(targets, stdouts):
                 self._display_stream(stdout, '[stdout:%i] ' % eid)
-        
+
             # republish stderr:
             for eid,stderr in zip(targets, stderrs):
                 self._display_stream(stderr, '[stderr:%i] ' % eid, file=sys.stderr)
-        
-            try:
-                get_ipython()
-            except NameError:
+
+            if get_ipython() is None:
                 # displaypub is meaningless outside IPython
                 return
             
