@@ -15,10 +15,12 @@ from traitlets import (
 )
 from decorator import decorator
 
+from IPython import get_ipython
 from ipyparallel import util
 from ipyparallel.controller.dependency import Dependency, dependent
 from ipython_genutils.py3compat import string_types, iteritems, PY3
 
+from ..serialize import PrePickled
 from . import map as Map
 from .asyncresult import AsyncResult, AsyncMapResult
 from .remotefunction import ParallelFunction, parallel, remote, getname
@@ -97,7 +99,7 @@ class View(HasTraits):
     results = Dict()
     client = Instance('ipyparallel.Client', allow_none=True)
 
-    _socket = Instance('zmq.Socket', allow_none=True)
+    _socket = Any()
     _flag_names = List(['targets', 'block', 'track'])
     _in_sync_results = Bool(False)
     _targets = Any()
@@ -552,9 +554,15 @@ class DirectView(View):
         
         _idents, _targets = self.client._build_targets(targets)
         futures = []
+
+        pf = PrePickled(f)
+        pargs = [PrePickled(arg) for arg in args]
+        pkwargs = {k: PrePickled(v) for k, v in kwargs.items()}
+
         for ident in _idents:
-            future = self.client.send_apply_request(self._socket, f, args, kwargs, track=track,
-                                    ident=ident)
+            future = self.client.send_apply_request(
+                self._socket, pf, pargs, pkwargs,
+                track=track, ident=ident)
             futures.append(future)
         if track:
             trackers = [_.tracker for _ in futures]
@@ -821,12 +829,9 @@ class DirectView(View):
         """
         
         from ipyparallel.client.magics import ParallelMagics
-        
-        try:
-            # This is injected into __builtins__.
-            ip = get_ipython()
-        except NameError:
-            print("The IPython parallel magics (%px, etc.) only work within IPython.")
+        ip = get_ipython()
+        if ip is None:
+            warnings.warn("The IPython parallel magics (%px, etc.) only work within IPython.")
             return
         
         M = ParallelMagics(ip, self, suffix)
@@ -990,7 +995,7 @@ class LoadBalancedView(View):
         """
 
         # validate whether we can run
-        if self._socket.closed:
+        if self._socket.closed():
             msg = "Task farming is disabled"
             if self._task_scheme == 'pure':
                 msg += " because the pure ZMQ scheduler cannot handle"
@@ -1127,7 +1132,7 @@ class LoadBalancedView(View):
         from joblib.parallel import register_parallel_backend
         from ._joblib import IPythonParallelBackend
         register_parallel_backend(name,
-            lambda : IPythonParallelBackend(view=self),
+            lambda **kwargs: IPythonParallelBackend(view=self, **kwargs),
             make_default=make_default)
 
 from concurrent.futures import Executor
