@@ -14,6 +14,7 @@ import os
 import sys
 import time
 
+from tornado.gen import coroutine, maybe_future
 import zmq
 from zmq.eventloop.zmqstream import ZMQStream
 
@@ -23,8 +24,9 @@ from ..util import extract_dates
 from jupyter_client.localinterfaces import localhost
 from ipython_genutils.py3compat import cast_bytes, unicode_type, iteritems, buffer_to_bytes_py2
 from traitlets import (
-        HasTraits, Any, Instance, Integer, Unicode, Dict, Set, Tuple, DottedObjectName
-        )
+    HasTraits, Any, Instance, Integer, Unicode, Dict, Set, Tuple,
+    DottedObjectName, observe
+)
 
 from ipyparallel import error, util
 from ipyparallel.factory import RegistrationFactory
@@ -213,7 +215,9 @@ class HubFactory(RegistrationFactory):
     db = Instance('ipyparallel.controller.dictdb.BaseDB', allow_none=True)
     heartmonitor = Instance('ipyparallel.controller.heartmonitor.HeartMonitor', allow_none=True)
 
-    def _ip_changed(self, name, old, new):
+    @observe('ip')
+    def _ip_changed(self, change):
+        new = change['new']
         self.engine_ip = new
         self.client_ip = new
         self.monitor_ip = new
@@ -222,7 +226,9 @@ class HubFactory(RegistrationFactory):
     def _update_monitor_url(self):
         self.monitor_url = "%s://%s:%i" % (self.monitor_transport, self.monitor_ip, self.mon_port)
 
-    def _transport_changed(self, name, old, new):
+    @observe('transport')
+    def _transport_changed(self, change):
+        new = change['new']
         self.engine_transport = new
         self.client_transport = new
         self.monitor_transport = new
@@ -522,6 +528,7 @@ class Hub(SessionFactory):
             self.log.error("Unrecognized monitor topic: %r", switch)
 
     @util.log_errors
+    @coroutine
     def dispatch_query(self, msg):
         """Route registration requests and queries from clients."""
         try:
@@ -556,7 +563,9 @@ class Hub(SessionFactory):
             return
         
         try:
-            handler(idents, msg)
+            f = handler(idents, msg)
+            if f:
+                yield maybe_future(f)
         except Exception:
             content = error.wrap_exception()
             self.log.error("Error handling request: %r", msg_type, exc_info=True)
@@ -586,10 +595,10 @@ class Hub(SessionFactory):
         triggers unregistration"""
         self.log.debug("heartbeat::handle_heart_failure(%r)", heart)
         eid = self.hearts.get(heart, None)
-        uuid = self.engines[eid].uuid
         if eid is None:
             self.log.info("heartbeat::ignoring heart failure %r (not an engine or already dead)", heart)
         else:
+            uuid = self.engines[eid].uuid
             self.unregister_engine(heart, dict(content=dict(id=eid, queue=uuid)))
 
     #----------------------- MUX Queue Traffic ------------------------------
@@ -1456,16 +1465,15 @@ class Hub(SessionFactory):
                                             parent=msg, ident=client_id,
                                             buffers=buffers)
 
+    @coroutine
     def become_dask(self, client_id, msg):
         """Start a dask.distributed Scheduler."""
         if self.distributed_scheduler is None:
             kwargs = msg['content'].get('scheduler_args', {})
             self.log.info("Becoming dask.distributed scheduler: %s", kwargs)
             from distributed import Scheduler
-            port = msg['content'].get('port', 0)
-            kwargs['loop'] = self.loop
             self.distributed_scheduler = scheduler = Scheduler(**kwargs)
-            f = scheduler.start(port)
+            yield scheduler.start()
         content = {
             'status': 'ok',
             'ip': self.distributed_scheduler.ip,
