@@ -9,11 +9,9 @@ Python Scheduler exists.
 # Distributed under the terms of the Modified BSD License.
 
 import logging
-import sys
 import time
 
 from collections import deque
-from datetime import datetime
 from random import randint, random
 from types import FunctionType
 
@@ -23,18 +21,18 @@ except ImportError:
     numpy = None
 
 import zmq
-from zmq.eventloop import ioloop, zmqstream
+from zmq.eventloop import zmqstream
 
 # local imports
 from decorator import decorator
 from traitlets.config.application import Application
 from traitlets.config.loader import Config
-from traitlets import Instance, Dict, List, Set, Integer, Enum, CBytes
+from traitlets import Instance, Dict, List, Set, Integer, Enum, CBytes, observe
 from ipython_genutils.py3compat import cast_bytes
 
 from ipyparallel import error, util
 from ipyparallel.factory import SessionFactory
-from ipyparallel.util import connect_logger, local_logger
+from ipyparallel.util import connect_logger, local_logger, ioloop
 
 from .dependency import Dependency
 
@@ -175,9 +173,11 @@ class TaskScheduler(SessionFactory):
 help="""select the task scheduler scheme  [default: Python LRU]
         Options are: 'pure', 'lru', 'plainrandom', 'weighted', 'twobin','leastload'"""
     )
-    def _scheme_name_changed(self, old, new):
-        self.log.debug("Using scheme %r"%new)
-        self.scheme = globals()[new]
+
+    @observe('scheme_name')
+    def _scheme_name_changed(self, change):
+        self.log.debug("Using scheme %r" % change['new'])
+        self.scheme = globals()[change['new']]
 
     # input arguments:
     scheme = Instance(FunctionType) # function for determining the destination
