@@ -1,22 +1,25 @@
 from .canning import (
     Reference, can_map, uncan_map, can, uncan,
-    use_dill, use_cloudpickle,
+    use_dill, use_cloudpickle, use_pickle,
 )
 from .serialize import (
     serialize_object, deserialize_object,
     pack_apply_message, unpack_apply_message,
+    PrePickled,
 )
 
 __all__ = (
     'Reference',
+    'PrePickled',
     'can_map',
     'uncan_map',
     'can',
     'uncan',
     'use_dill',
     'use_cloudpickle',
+    'use_pickle',
     'serialize_object',
     'deserialize_object',
     'pack_apply_message',
     'unpack_apply_message',
-)
\ No newline at end of file
+)
