@@ -12,7 +12,10 @@ import os
 import sys
 from threading import Thread
 import time
-import mock
+try:
+    from unittest import mock
+except ImportError:
+    import mock
 
 import pytest
 import tornado
@@ -554,8 +557,8 @@ class TestClient(ClusterTestCase):
 
     @skip_without('distributed')
     @pytest.mark.skipif(
-        sys.version_info[:2] == (3, 4),
-        reason="become_dask doesn't work on Python 3.4")
+        sys.version_info[:2] <= (3, 5),
+        reason="become_dask doesn't work on Python 3.5")
     @pytest.mark.skipif(
         tornado.version_info[:2] < (5,),
         reason="become_dask doesn't work with tornado 4")
@@ -586,12 +589,12 @@ class TestClient(ClusterTestCase):
             with mock.patch('socket.gethostname', lambda: location), \
                     pytest.warns(None) as record:  # should not trigger warning
                 c = self.connect_client()
-                assert len(record) == 0, str(record)
+                assert len(record) == 0, str([str(w) for w in record])
                 c.close()
 
     def test_local_ip_true_doesnt_trigger_warning(self):
         with mock.patch('ipyparallel.client.client.is_local_ip',
                         lambda x: True), pytest.warns(None) as record:
             c = self.connect_client()
-            assert len(record) == 0, str(record)
+            assert len(record) == 0, str([str(w) for w in record])
             c.close()
